"""
shōmei - a CLI tool that safely updates your personal github graph to reflect the work you did from a corp github account, without ever exposing proprietary code or company IP.

transforms your commits into safe, sanitized commits, and publishes them to your personal github profile, so your contribution graph reflects your real effort.
"""

__version__ = "0.2.5"
__author__ = "shomei contributors"
__description__ = "update your personal github graph to reflect the work you did"
